# -*- coding: utf-8 -*-
"""
Created on Mon Sep 20 22:57:26 2021

@author: mumuz
"""

import os
from simuwater.simuwater import *
from simuwater.swmmnodes import *
from simuwater.swmmlinks import *

class Simulation(object):
    def __init__(self,
                 swi,
                 swr = None,
                 swo = None,
                 swmm_cpy_dll = ''):
        self._model = Simuwater(swmm_cpy_dll)
        if swr == None:
            swr = os.path.dirname(swi) + '\\' + self.__path_getFileName(swi) + '.swr'
        if swo == None:
            swo = os.path.dirname(swi) + '\\' + self.__path_getFileName(swi) + '.swo'
        self._model.simuwater_open(swi, swr, swo)
        self._isOpen = True
        self._advance_seconds = None
        self._isStarted = False
        self._terminate_request = False
        self._callbacks = {
            "before_start": None,
            "before_step": None,
            "after_step": None,
            "before_end": None,
            "after_end": None,
            "after_close": None
        }
    
    def __path_getFileName(self, file_name, with_suffix = False):
        if with_suffix:
            return os.path.split(file_name)[-1]
        else:
            return os.path.split(file_name)[-1].split('.')[0]
        
    def __enter__(self):
        return self

    def __iter__(self):
        return self

    def start(self):
        if not self._isStarted:
            if hasattr(self, "_initial_conditions"):
                self._initial_conditions()
            self._execute_callback(self.before_start())
            self._model.simuwater_start()
            self._isStarted = True

    def __next__(self):
        self.start()
        if self._terminate_request:
            self._execute_callback(self.before_end())
            raise StopIteration
        self._execute_callback(self.before_step())
        if self._advance_seconds is None:
            err_code, time = self._model.simuwater_step()
        else:
            err_code, time = self._model.simuwater_stride(self._advance_seconds)
        self._execute_callback(self.after_step())
        if time <= 0.0:
            self._execute_callback(self.before_end())
            raise StopIteration
        return self._model

    next = __next__  # Python 2

    def __exit__(self, *a):
        if self._isStarted:
            self._model.simuwater_end()
            self._isStarted = False
            self._execute_callback(self.after_end())
        if self._isOpen:
            self._model.simuwater_close()
            self._isOpen = False
            self._execute_callback(self.after_close())

    @staticmethod
    def _is_callback(callable_object):
        if not callable(callable_object):
            error_msg = 'Requires Callable Object, not {}'.format(
                type(callable_object))
            raise (PySimuwaterException(error_msg))
        else:
            return True

    def _execute_callback(self, callback):
        if callback:
            try:
                callback()
            except PySimuwaterException:
                error_msg = "Callback Failed"
                raise PySimuwaterException((error_msg))

    def initial_conditions(self, init_conditions):
        if self._is_callback(init_conditions):
            self._initial_conditions = init_conditions

    def before_start(self):
        return self._callbacks["before_start"]

    def add_before_start(self, callback):
        if self._is_callback(callback):
            self._callbacks["before_start"] = callback

    def before_step(self):
        return self._callbacks["before_step"]

    def add_before_step(self, callback):
        if self._is_callback(callback):
            self._callbacks["before_step"] = callback

    def after_step(self):
        return self._callbacks["after_step"]

    def add_after_step(self, callback):
        if self._is_callback(callback):
            self._callbacks["after_step"] = callback

    def before_end(self):
        return self._callbacks["before_end"]

    def add_before_end(self, callback):
        if self._is_callback(callback):
            self._callbacks["before_end"] = callback

    def after_end(self):
        return self._callbacks["after_end"]

    def add_after_end(self, callback):
        if self._is_callback(callback):
            self._callbacks["after_end"] = callback

    def after_close(self):
        return self._callbacks["after_close"]

    def add_after_close(self, callback):
        if self._is_callback(callback):
            self._callbacks["after_close"] = callback

    def step_advance(self, advance_seconds):
        self._advance_seconds = advance_seconds

    def terminate_simulation(self):
        self._terminate_request = True

    def report(self):
        self._model.simuwater_report()

    def close(self):
        self.__exit__()

    def execute(self):
        self._model.simuwater_run()

    @property
    def runoff_error(self):
        return self._model.simuwater_getMassBalErr()[1]

    @property
    def flow_routing_error(self):
        return self._model.simuwater_getMassBalErr()[2]

    @property
    def quality_error(self):
        return self._model.simuwater_getMassBalErr()[3]

    @property
    def start_time(self):
        return self._model.simuwater_getDateTime()[1]

    @property
    def end_time(self):
        return self._model.simuwater_getDateTime()[2]

    @property
    def current_time(self):
        return self._model.simuwater_getCurrentTime()

    @property
    def percent_complete(self):
        dt = self.current_time - self.start_time
        total_time = self.end_time - self.start_time
        return float(dt.total_seconds()) / total_time.total_seconds()
    
    # 测试
if __name__ == '__main__':
    swi = r'C:\Users\mumuz\Desktop\simuwater\python\example\medium.swi'
    swr = r'C:\Users\mumuz\Desktop\simuwater\python\example\medium.swr'
    swo = r'C:\Users\mumuz\Desktop\simuwater\python\example\medium.swo'
    sim = Simulation(swi, swr, swo)
    sim.step_advance(300)
    for step in sim:
#        for node in SwmmNodes(sim, 'SWMM2'):
#            print(node.node_id)
#            print(node.inflow)
#            break
        swmm_link = SwmmLink(sim,'SWMM2','P3')
#        swmm_link.set_setting(1.0)
        print(swmm_link.flow)
#        for link in SwmmLinks(sim, 'SWMM2'):
#            print(link.link_id)
#            print(link.flow)
#            break
#        print(sim.percent_complete)
    sim.close()
    print("run is successful.")


# -*- coding: utf-8 -*-
"""
Created on Thu Sep 23 11:15:56 2021

@author: mumuz
"""

from simuwater.simuwater import PySimuwaterException

class SwmmNodes(object):
    
    def __init__(self, sim, inp_id):
        if not sim._isStarted:
            raise PySimuwaterException('SWMM Model Within Simuwater Not Open')
        self._sim = sim
        self._inp_id = inp_id
        self._model = sim._model
        self._cur_index = 0
        self._nNodes = self._model.simuwater_getSwmmObjCount(self._inp_id, 1)
    
    def __len__(self):
        return self._model.simuwater_getSwmmObjCount(self._inp_id, 1)
    
    def __contains__(self, node_id):
        if self._model.simuwater_getSwmmObjIndex(self._inp_id, 1,node_id) < 0:
            return False
        return True
    
    def __getitem__(self, node_id):
        if self.__contains__(node_id):
            return SwmmNode(self._sim, self._inp_id, node_id)
        else:
            raise PySimuwaterException('Node ID: {} Does not Exists'.format(node_id))
    
    def __iter__(self):
        return self
    
    def __next__(self):
        if self._cur_index < self._nNodes:
            node_obj = self.__getitem__(self._node_id)
            self._cur_index += 1
            return node_obj
        else:
            raise StopIteration()
    
    next = __next__  # Python 2
    
    @property
    def _node_id(self):
        return self._model.simuwater_getSwmmObjID(self._inp_id, 1, self._cur_index)[1]
        
class SwmmNode(object):
    
    def __init__(self, sim, inp_id, node_id):
        if not sim._isStarted:
            raise PySimuwaterException('SWMM Model Within Simuwater Not Open')
        if sim._model.simuwater_getSwmmObjIndex(inp_id, 1, node_id) < 0:
            raise PySimuwaterException('ID Invalid')
        self._sim = sim
        self._model = sim._model
        self._inp_id = inp_id
        self._node_id = node_id
        
    @property
    def node_id(self): 
        return self._node_id
    
    @property
    def depth(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 1, self._node_id, 0)[1]
    
    @property
    def head(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 1, self._node_id, 1)[1]
    
    @property
    def volume(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 1, self._node_id, 2)[1]
    
    @property
    def lateral_inflow(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 1, self._node_id, 3)[1]
    
    @property
    def total_inflow(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 1, self._node_id, 4)[1]
   
    @property
    def overflow(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 1, self._node_id, 5)[1]
    
    def set_external_inflow(self, ext_inflow):
        return self._model.simuwater_setSwmmExtInflow(self._inp_id, self._node_id, ext_inflow)


# -*- coding: utf-8 -*-
"""
Created on Sat Oct  9 10:47:59 2021

@author: mumuz
"""

import ctypes
import os

LIB_PATH = os.path.abspath(os.path.dirname(__file__))

SIMUWATER = ctypes.cdll.LoadLibrary(LIB_PATH + '\\simuwater.dll')
OUTPUT = ctypes.cdll.LoadLibrary(LIB_PATH + '\\simuwater_output.dll')


# -*- coding: utf-8 -*-
"""
Created on Sat Oct  9 12:42:59 2021

@author: mumuz
"""

import ctypes
import os
from simuwater.lib import *
import datetime


# 对象类型索引
SYSTEM = 0
SUBCATCH = 1
JUNCTION = 2
TANK = 3
OUTFALL = 4
SPLITTER = 5
REACTOR = 6
LID_MONO = 7
PIPE = 8
PUMP = 9
CONNECTION = 10
CONDUIT = 11
WEIR = 12

# 系统结果类型
SYSTEM_RAINFALL = 0
SYSTEM_EVAP = 1
SYSTEM_INFIL = 2
SYSTEM_RUNOFF = 3
SYSTEM_TOT_INFLOW = 4
SYSTEM_DIR_INFLOW = 5
SYSTEM_DWF = 6
SYSTEM_FLOOD = 7
SYSTEM_SWMM_INFLOW = 8
SYSTEM_OUTFLOW = 9
SYSTEM_SWMM_OUTFLOW = 10
SYSTEM_STORAGE = 11

# 子汇水区结果类型
SUBCATCH_RAINFALL = 0
SUBCATCH_EVAP = 1
SUBCATCH_INFIL = 2
SUBCATCH_RUNOFF = 3
SUBCATCH_QUAL = 4

# junction节点结果类型
JUNCTION_TOT_INFLOW = 0
JUNCTION_LAT_INFLOW = 1
JUNCTION_QUAL = 2

# tank节点结果类型
TANK_DEPTH = 0
TANK_VOL = 1
TANK_TOT_INFLOW = 2
TANK_LAT_INFLOW = 3
TANK_FLOOD = 4
TANK_QUAL = 5

# outfall节点结果类型
OUTFALL_TOT_INFLOW = 0
OUTFALL_LAT_INFLOW = 1
OUTFALL_QUAL = 2

# splitter节点结果类型
SPLITTER_TOT_INFLOW = 0
SPLITTER_LAT_INFLOW = 1
SPLITTER_OUTFLOW1 = 2
SPLITTER_OUTFLOW2 = 3
SPLITTER_QUAL = 4

# reactor节点结果类型
REACTOR_VOL = 0
REACTOR_TOT_INFLOW = 1
REACTOR_LAT_INFLOW = 2
REACTOR_OUTFLOW = 3
REACTOR_SURPASS = 4
REACTOR_EVAP = 5
REACTOR_INFIL = 6
REACTOR_EMPTY = 7
REACTOR_QUAL = 8

# lidmono节点结果类型
LID_MONO_TOT_INFLOW = 0
LID_MONO_LAT_INFLOW = 1
LID_MONO_RAINFALL = 2
LID_MONO_EVAP = 3
LID_MONO_SURF_OUTFLOW = 4
LID_MONO_STORAGE_EXFIL = 5
LID_MONO_STORAGE_DRAIN = 6
LID_MONO_SURF_DEPTH = 7
LID_MONO_SOIL_MOIST = 8
LID_MONO_STORAGE_DEPTH = 9
LID_MONO_PAVE_DEPTH = 10
LID_MONO_QUAL = 11

# pipe链接结果类型
PIPE_VOL = 0
PIPE_INFLOW = 1
PIPE_OUTFLOW = 2
PIPE_FLOOD = 3
PIPE_QUAL = 4

# pump链接结果类型
PUMP_FLOW = 0
PUMP_QUAL = 1

# connection链接结果类型
CONNECTION_INFLOW = 0
CONNECTION_OUTFLOW = 1
CONNECTION_QUAL = 2

# conduit链接结果类型
CONDUIT_DEPTH = 0
CONDUIT_VOL = 1
CONDUIT_VELOCITY = 2
CONDUIT_INFLOW = 3
CONDUIT_OUTFLOW = 4
CONDUIT_FLOOD = 5
CONDUIT_QUAL = 6

# weir链接结果类型
WEIR_FLOW = 0
WEIR_QUAL = 1


class Output():
    
    def __init__(self):
        self._output = OUTPUT
        self._is_open = False
        self._n_period = 0
        self._start_date_time = 0
        self._report_step = 0
        self._c_double_type = ctypes.POINTER(ctypes.c_double)
        self._c_float_type = ctypes.POINTER(ctypes.c_float)
        self._c_int_type = ctypes.POINTER(ctypes.c_int)
        
    @property
    def is_open(self):
        return self._is_open
    
    @property
    def n_period(self):
        return self._n_period
    
    @property
    def start_date_time(self):
        return self.__time_fromReal(self._start_date_time)
    
    @property
    def report_step(self):
        return self._report_step
        
    def open(self, swo_path):
        err_code = self._output.output_openSwoFile(swo_path.encode('utf-8'))
        if err_code == 0:
            n_period = 0    # 报告周期数量
            start_date_time = 0.0    # 报告开始时间（浮点型日期格式）
            report_step = 0    # 报告步长（sec）
            p_n_period = self._c_int_type(ctypes.c_int(n_period))
            p_start_date_time = self._c_double_type(ctypes.c_double(start_date_time))
            p_report_step = self._c_int_type(ctypes.c_int(report_step))
            self._output.output_getProjectParams(p_n_period, p_start_date_time, p_report_step)
            n_period = p_n_period[0]
            start_date_time = p_start_date_time[0]
            report_step = p_report_step[0]
            self._n_period = n_period
            self._start_date_time = start_date_time
            self._report_step = report_step
            self._is_open = True
            return True
        else:
            self._is_open = False
            return False
    
    def get_result(self, obj_type, name, var_type, period):
        flag = False
        value = 0
        p_value = self._c_float_type(ctypes.c_float(value))
        if not self._is_open:
            return False, value
        idx = self._output.output_getIndex(obj_type - 1, name.encode('utf-8'))
        print(idx)
        err_code = self._output.output_getSimuwaterResult(obj_type, idx, var_type, period, p_value)
        value = p_value[0]
        if err_code == 0:
            flag = True
        else:
            flag = False
        return flag, value

    def get_results(self, obj_type, name, var_type):
        if not self._is_open:
            return False, None
        results = []
        for i in range(1, self.n_period + 1):
            flag, result = self.get_result(obj_type, name, var_type, i)
            if not flag:
                return False, None
            results.append(result)
        return results
    
    def close(self):
        self._output.output_closeSwoFile()
        
        # 数值型时间转日期型时间
    # private
    def __time_fromReal(self, time):
        return datetime.datetime.strptime('1899-12-30','%Y-%m-%d') + datetime.timedelta(time)
        


# -*- coding: utf-8 -*-
"""
Created on Sat Oct  9 14:12:41 2021

@author: mumuz
"""

import setuptools

setuptools.setup(name='simuwater',
                 version='0.0.3',
                 description='Simuwater is a control-oriented modelling software',
                 url='',
                 author='Lei Zhang',
                 author_email='gemini.zhang@qq.com',
                 license='MIT',
                 packages=setuptools.find_packages(),
				 package_data={'':['*.dll'],})


# -*- coding: utf-8 -*-
"""
Created on Thu Sep 23 13:33:04 2021

@author: mumuz
"""


from simuwater.simuwater import PySimuwaterException

class SwmmLinks(object):
    
    def __init__(self, sim, inp_id):
        if not sim._isStarted:
            raise PySimuwaterException('SWMM Model Within Simuwater Not Open')
        self._sim = sim
        self._inp_id = inp_id
        self._model = sim._model
        self._cur_index = 0
        self._nLinks = self._model.simuwater_getSwmmObjCount(self._inp_id, 2)
    
    def __len__(self):
        return self._model.simuwater_getSwmmObjCount(self._inp_id, 2)
    
    def __contains__(self, link_id):
        if self._model.simuwater_getSwmmObjIndex(self._inp_id, 2,link_id) < 0:
            return False
        return True
    
    def __getitem__(self, link_id):
        if self.__contains__(link_id):
            return SwmmLink(self._sim, self._inp_id, link_id)
        else:
            raise PySimuwaterException('Link ID: {} Does not Exists'.format(link_id))
    
    def __iter__(self):
        return self
    
    def __next__(self):
        if self._cur_index < self._nLinks:
            link_obj = self.__getitem__(self._link_id)
            self._cur_index += 1
            return link_obj
        else:
            raise StopIteration()
    
    next = __next__  # Python 2
    
    @property
    def _link_id(self):
        return self._model.simuwater_getSwmmObjID(self._inp_id, 2, self._cur_index)[1]
        
class SwmmLink(object):
    
    def __init__(self, sim, inp_id, link_id):
        if not sim._isStarted:
            raise PySimuwaterException('SWMM Model Within Simuwater Not Open')
        if sim._model.simuwater_getSwmmObjIndex(inp_id, 2, link_id) < 0:
            raise PySimuwaterException('ID Invalid')
        self._sim = sim
        self._model = sim._model
        self._inp_id = inp_id
        self._link_id = link_id
        
    @property
    def link_id(self): 
        return self._link_id
    
    @property
    def flow(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 2, self._link_id, 0)[1]
    
    @property
    def depth(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 2, self._link_id, 1)[1]
    
    @property
    def velocity(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 2, self._link_id, 2)[1]
    
    @property
    def volume(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 2, self._link_id, 3)[1]
    
    @property
    def capacity(self):
        return self._model.simuwater_getSwmmResult(self._inp_id, 2, self._link_id, 4)[1]
    
    def set_setting(self,setting):
        return self._model.simuwater_setSwmmSetting(self._inp_id, self._link_id, setting)